import argparse
import datetime
import os
import queue
import socket
import sys
import threading

from elasticsearch_raven.transport import ElasticsearchTransport
from elasticsearch_raven.transport import SentryMessage


def run_server():
    args = _parse_args()
    sock = get_socket(args.ip, args.port)
    if sock:
        _run_server(sock)


def _parse_args():
    parser = argparse.ArgumentParser(description='Udp proxy server for raven')
    parser.add_argument('ip')
    parser.add_argument('port', type=int)
    return parser.parse_args()


def get_socket(ip, port):
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        sock.bind((ip, int(port)))
        return sock
    except socket.gaierror:
        sys.stdout.write('Wrong hostname\n')


def _run_server(sock):
    blocking_queue = queue.Queue(maxsize=os.environ.get('QUEUE_MAXSIZE', 1000))
    sender = _get_sender(blocking_queue)
    _serve(sock, blocking_queue, sender)


def _serve(sock, blocking_queue, sender):
    sender.start()
    while True:
        data, address = sock.recvfrom(65535)
        message = SentryMessage.create_from_udp(data)
        blocking_queue.put(message)
        sys.stdout.write('{host}:{port} [{date}]\n'.format(
            host=address[0], port=address[1], date=datetime.datetime.now()))


def _get_sender(blocking_queue):
    host = os.environ.get('ELASTICSEARCH_HOST', 'localhost:9200')
    use_ssl = bool(os.environ.get('USE_SSL', False))
    transport = ElasticsearchTransport(host, use_ssl)

    def _send():
        while True:
            message = blocking_queue.get()
            transport.send(message)
            blocking_queue.task_done()

    sender = threading.Thread(target=_send)
    return sender



import base64
import datetime
import json
import re
import zlib

import elasticsearch
from elasticsearch_raven.postfix import postfix_encoded_data


class Message:
    def __init__(self, headers, body):
        self._body = body
        self._headers = headers

    @classmethod
    def from_message(cls, message):
        return cls(message.headers, message.body)

    @property
    def body(self):
        return self._body

    @property
    def headers(self):
        return self._headers


class SentryMessage(Message):
    def __init__(self, headers, body):
        super().__init__(headers, body)
        self._compressed = True

    @classmethod
    def create_from_udp(cls, data):
        byte_headers, data = data.split(b'\n\n')
        headers = cls.parse_headers(str(byte_headers.decode('utf-8')))
        data = base64.b64decode(data)
        return cls(headers, data)

    @staticmethod
    def parse_headers(unparsed_headers):
        m = re.search(r'sentry_key=(?P<sentry_key>[^=]+), sentry_secret='
                      r'(?P<sentry_secret>[^=]+)$', unparsed_headers)
        return m.groupdict()


    @classmethod
    def create_from_http(cls, unparsed_headers, data):
        headers = cls.parse_headers(unparsed_headers)
        data = base64.b64decode(data)
        return cls(headers, data)

    @property
    def body(self):
        if self._compressed:
            self._body = json.loads(
                zlib.decompress(self._body).decode('utf-8'))
            self._compressed = False
        return self._body


class ElasticsearchMessage(Message):
    def __init__(self, headers, body):
        super().__init__(headers, body)
        postfix_encoded_data(self._body)


class ElasticsearchTransport:
    def __init__(self, host, use_ssl=False):
        self._host = host
        self._use_ssl = use_ssl

    def send(self, message):
        message = ElasticsearchMessage.from_message(message)
        dated_index = message.body['project'].format(datetime.datetime.now())
        http_auth = '{}:{}'.format(message.headers['sentry_key'],
                                   message.headers['sentry_secret'])
        connection = elasticsearch.Elasticsearch(hosts=[self._host],
                                                 http_auth=http_auth,
                                                 use_ssl=self._use_ssl)
        connection.index(body=message.body, index=dated_index,
                         doc_type='raven-log')



from setuptools import setup


setup(
    name='Elasticsearch-Raven',
    version='0.1.1',
    author='Tomasz Wysocki',
    author_email='tomasz@pozytywnie.pl',
    packages=['elasticsearch_raven'],
    scripts=['bin/elasticsearch-raven.py'],
    license='MIT',
    description='Proxy that allows to log from raven to elasticsearch.',
    long_description=open('README.md').read(),
    install_requires=['elasticsearch'],
    test_suite='tests'
)
